#
# Copyright (C) 2018  FreeIPA Contributors see COPYING for license
#
from ipaplatform.paths import paths
from ipapython import sysrestore
from ipapython import ipautil
from ipapython.ipa_log_manager import root_logger
from ipaserver.install import service
from ipapython import ntpmethods
from ipapython.ntpmethods import detect_time_server


class BaseNTPServer(service.Service):
    timeout = 15

    def __init__(self, service_name, ntp_confile=None, fstore=None,
                 ntp_bin=None, ntp_servers=None, ntp_pool=None,
                 sstore=None, flag=None, opts=None):
        super(BaseNTPServer, self).__init__(
            service_name=service_name,
            service_desc="NTP daemon",
            sstore=sstore,
        )

        self.ntp_confile = ntp_confile
        self.ntp_servers = ntp_servers
        self.ntp_pool = ntp_pool
        self.ntp_bin = ntp_bin
        self.flag = flag
        self.fstore = fstore
        self.opts = opts

        self.args = [paths.BIN_TIMEOUT, str(self.timeout),
                     self.ntp_bin, self.flag]

        if not fstore:
            self.fstore = sysrestore.FileStore(paths.SYSRESTORE)

    def __configure_ntp(self):
        root_logger.debug("Backing up %s", self.ntp_confile)
        self.fstore.backup_file(self.ntp_confile)

        root_logger.debug("Configuring %s", detect_time_server())

        if not self.ntp_servers and not self.ntp_pool:
            self.ntp_pool = "pool.ntp.org"

        config_content = ntpmethods.set_config(
            path=self.ntp_confile,
            pool=self.ntp_pool,
            servers=self.ntp_servers,
            opts=self.opts,
        )

        root_logger.debug("Writing configuration to %s", self.ntp_confile)

        ntpmethods.service_command()['api'].stop()
        ntpmethods.write_config(self.ntp_confile, config_content)

    def __start_sync(self):
        root_logger.debug("Sync time with %s", detect_time_server())
        ipautil.run(self.args)

    def sync_time(self):
        self.step("stopping {}".format(self.service_name), self.stop)
        self.step("writing configuration", self.__configure_ntp)
        self.step("configuring {} to start on boot".format(self.service_name), self.enable)
        self.step("synchronization time with {}".format(detect_time_server()), self.__start_sync)
        self.step("starting {}".format(self.service_name), self.start)

        self.start_creation()

    def uninstall(self):
        ntpmethods.uninstall(
            self.sstore, self.fstore, self.ntp_confile, root_logger
        )
